"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withExtendableCaching = void 0;
const _1 = require(".");
const isAFunction_1 = require("./isAFunction");
const withSimpleCaching_1 = require("./withSimpleCaching");
/**
 * exposes the cache-wrapped method along with some primitives which enable extending the caching logic
 *
 * specifically
 * - exposes a way to `invalidate` the cache, for a given input (e.g., to support external triggers for invalidation)
 * - exposes a way to `update` the cache, for a given input (e.g., to support write-through caching and optimistic caching)
 *
 * relevance
 * - when wrapping logic to cache the user is able to specify several caching options (e.g., key serialization method, value serialization method, etc)
 * - in order to define their own `invalidation` and `update` methods, without this function, the user would need to access these caching options per function elsewhere
 * - this function makes it easy to utilize and extend cache invalidation + update commands for the wrapped logic, by managing the references to the caching options on behalf of the user
 */
const withExtendableCaching = (logic, options) => {
    const execute = (0, _1.withSimpleCaching)(logic, options);
    const invalidate = ({ forInput }) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c;
        const serializeKey = (_b = (_a = options.serialize) === null || _a === void 0 ? void 0 : _a.key) !== null && _b !== void 0 ? _b : withSimpleCaching_1.defaultKeySerializationMethod;
        const cache = (0, withSimpleCaching_1.getCacheFromCacheOption)({ forInput, cacheOption: options.cache });
        yield cache.set(serializeKey(forInput), undefined);
        if ((_c = options.hook) === null || _c === void 0 ? void 0 : _c.onSet)
            // note: we do not wait for the hook to resolve; hooks do not block execution ℹ️
            options.hook.onSet({
                trigger: _1.WithSimpleCachingOnSetTrigger.INVALIDATE,
                forInput,
                forKey: serializeKey(forInput),
                value: undefined,
            });
    });
    const update = ({ forInput, toValue }) => __awaiter(void 0, void 0, void 0, function* () {
        var _d, _e, _f, _g, _h;
        const serializeKey = (_e = (_d = options.serialize) === null || _d === void 0 ? void 0 : _d.key) !== null && _e !== void 0 ? _e : withSimpleCaching_1.defaultKeySerializationMethod;
        const serializeValue = (_g = (_f = options.serialize) === null || _f === void 0 ? void 0 : _f.value) !== null && _g !== void 0 ? _g : withSimpleCaching_1.defaultValueSerializationMethod;
        const cache = (0, withSimpleCaching_1.getCacheFromCacheOption)({ forInput, cacheOption: options.cache });
        const newValue = (0, isAFunction_1.isAFunction)(toValue) ? toValue({ cachedValue: yield cache.get(serializeKey(forInput)) }) : toValue;
        yield cache.set(serializeKey(forInput), serializeValue(newValue), { secondsUntilExpiration: options.secondsUntilExpiration });
        if ((_h = options.hook) === null || _h === void 0 ? void 0 : _h.onSet)
            // note: we do not wait for the hook to resolve; hooks do not block execution ℹ️
            options.hook.onSet({
                trigger: _1.WithSimpleCachingOnSetTrigger.UPDATE,
                forInput,
                forKey: serializeKey(forInput),
                value: {
                    output: newValue,
                    cached: serializeValue(newValue),
                },
            });
    });
    return { execute, invalidate, update };
};
exports.withExtendableCaching = withExtendableCaching;
//# sourceMappingURL=withExtendableCaching.js.map